<?php

namespace Pinpoint\CronRunner\Controller\Adminhtml\Crons;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;

/**
 * Class RunJob
 * @package Pinpoint\CronRunner\Controller\Adminhtml\Crons
 */
class RunJob extends Action
{
    /**
     * @var \Magento\Cron\Model\Config
     */
    protected $config;

    /**
     * RunJob constructor.
     * @param Context $context
     * @param \Magento\Cron\Model\Config $config
     */
    public function __construct(
        Context $context,
        \Magento\Cron\Model\Config $config
    ) {
        parent::__construct($context);
        $this->config = $config;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface
     */
    public function execute()
    {
        $cron = $this->getCronInfo($this->getRequest()->getParam('jobcode'));
        if ($cron) {
            $class = $this->_objectManager->create($cron['instance']);
            $time = time();

            try {
                $class->{$cron['method']}();
                $this->messageManager->addSuccess(sprintf("Cron '%s' ran. Took %s second(s).", $cron['name'], time() - $time));
            } catch (\Exception $exception) {
                $this->messageManager->addNotice(sprintf("Cron '%s' ran and but returned an error. Took %s second(s).", $cron['name'], time() - $time));
                $this->messageManager->addNotice($exception->getMessage());
            }
        }

        return $this->_redirect('*/*/list');
    }

    /**
     * @param $jobCode
     * @return null
     */
    protected function getCronInfo($jobCode)
    {
        foreach ($this->config->getJobs() as $section => $jobs) {
            foreach ($jobs as $job) {
                if ($job['name'] == $jobCode) {
                    return $job;
                }
            }
        }

        return null;
    }
}
